import unittest

from unittest.mock import patch

from datetime import datetime
from dateintelligence import CalendarOperations

class CalendarOperationsTestCase(unittest.TestCase):
    def setUp(self):
        self.out = CalendarOperations()
        self.base_date = datetime(2018, 5, 5)

    def test_default_widget_size(self):
        pass

    def test_valid_add_months(self):
        actual_result = self.out.add_months(self.base_date, 3)
        expected_result = datetime(2018, 8, 5)
        self.assertEqual(actual_result,expected_result)

    def test_valid_add_months_to_next_year(self):
        actual_result = self.out.add_months(self.base_date, 8)
        expected_result = datetime(2019, 1, 5)
        self.assertEqual(actual_result,expected_result)

    def test_valid_add_months_zero_months(self):
        actual_result = self.out.add_months(self.base_date, 0)
        expected_result = datetime(2018, 5, 5)
        self.assertEqual(actual_result,expected_result)

    def test_valid_add_months_negative_months(self):
        actual_result = self.out.add_months(self.base_date, -1)
        expected_result = datetime(2018, 4, 5)
        self.assertEqual(actual_result,expected_result)

    def test_add_months_not_a_date(self):
        self.assertRaises(ValueError, self.out.add_months, "not a date", 1)

    def test_add_months_not_an_int(self):
        self.assertRaises(ValueError, self.out.add_months, self.base_date, "not an int")

    '''
    @patch('dateintelligence.calendaroperations.datetime')
    def test_last_day_of_month(self, mock_dt):
        mock_dt.now.return_value = self.base_date

        actual_result = self.out.last_day_of_month()
        expected_result = datetime(2018,5,31)
        self.assertEqual(actual_result,expected_result)
    '''


import openpyxl


class ExcelWriter:
    def __init__(self, filename):
        self.workbook = openpyxl.Workbook()
        self.filename = filename

    def create_sheet(self, sheetname):
        return self.workbook.create_sheet(index=0, title=sheetname)

    def build_sheet(self, sheetname, sheet_data):
        sheet = self.workbook.create_sheet(index=0, title=sheetname)
        for key, value in sheet_data.items():
            sheet[key] = value

    def save(self):
        self.workbook.save(filename=self.filename)



import argparse
import os
import sys
import time
from datetime import datetime
import pkg_resources

import credisur.datastructures as datastructures
import credisur.dateintelligence as dateintelligence
import credisur.exceladapter as exceladapter
import credisur.excelbuilder as excelbuilder
import credisur.tableextraction as tableextraction
from credisur.config import \
    get_columns_configuration, \
    get_advance_payments_columns, \
    get_last_payment_columns, \
    get_no_payment_due_columns
from credisur.extractors import customer_row_extractor

from credisur.models import AccountReceivable

# TODO: Copiar listado de facturas en solapa.
# TODO: Controlar consistencia código (por ejemplo, D-E está mal.
# TODO: Resolver punitorio


def working_directory():
    return os.getcwd().replace('\\', '/')

def valid_date(s):
    try:
        return datetime.strptime(s, "%Y-%m-%d")
    except ValueError:
        msg = "No es una fecha válida: '{0}'.".format(s)
        raise argparse.ArgumentTypeError(msg)

def get_parser():
    parser = argparse.ArgumentParser(description='Script para procesar planillas de Credisur')
    parser.add_argument("--version", "-v", help="Muestra la versión.", action="store_true")
    parser.add_argument("--inputs", "-i", help="Permite definir la carpeta de inputs", default="inputs")
    parser.add_argument("--outputs", "-o", help="Permite definir la carpeta de outputs", default="outputs")
    parser.add_argument(
        "--date","-d",
        help="Permite definir la fecha de cálculo para vencimientos. Formato: AAAA-MM-DD. Si no se especifica, toma la fecha de hoy.",
        type=valid_date
    )

    return parser


def is_code_permitted(code):
    return True


def main(args=None):
    PAYMENT_ERRORS = []

    """The main routine."""
    if args is None:
        args = sys.argv[1:]

    cwd = working_directory()

    parser = get_parser()
    params = parser.parse_args(args)

    if params.version:
        print(pkg_resources.require("credisuretl")[0].version)
        return

    inputs_path = "%s/%s/" % (cwd, params.inputs)
    outputs_path = "%s/%s/" % (cwd, params.outputs)

    # abrir archivos
    NUEVO = "nuevo"
    HISTORICO = "histórico"

    calendar_ops = dateintelligence.CalendarOperations()

    # calendar
    current_date = params.date or datetime.now()
    first_day_of_current_month = datetime(current_date.year, current_date.month, 1)
    last_date_of_month = calendar_ops.last_day_of_month(current_date)

    def get_old_excel_filename(filename):
        return filename[:-1]

    def upgrade_if_older_version(filename):
        old_filename = get_old_excel_filename(filename)
        if not os.path.isfile(filename) and os.path.isfile(old_filename):
            exceladapter.ExcelUpgrader(old_filename).upgrade()

    errors = []
    collections = datastructures.HashOfLists()
    collections_for_customers = datastructures.HashOfLists()
    bills = {}

    customers_in_last_payment = []
    customers_without_payments_due = []
    list_of_advance_payments = []

    accounts_to_collect = {
        "C": [],
        "D": [],
        "I": []
    }

    input_customers_filename = inputs_path + 'Clientes.xlsx'
    input_collections_filename = inputs_path + 'Cobranza.xlsx'
    input_pending_bills_filename = inputs_path + 'Factura.xlsx'
    input_accounts_to_collect_filename = inputs_path + 'Cuentas a Cobrar.xlsx'

    upgrade_if_older_version(input_customers_filename)
    upgrade_if_older_version(input_collections_filename)
    upgrade_if_older_version(input_pending_bills_filename)
    upgrade_if_older_version(input_accounts_to_collect_filename)

    # ExcelReader debería tomar un Stream en vez de un filename - TODO: probar
    collections_reader = exceladapter.excelreader.ExcelReader(input_collections_filename)
    pending_bills_reader = exceladapter.excelreader.ExcelReader(input_pending_bills_filename)
    accounts_to_collect_reader = exceladapter.excelreader.ExcelReader(input_accounts_to_collect_filename)

    collections_sheet = collections_reader.get_sheet('hoja1')
    pending_bills_sheet = pending_bills_reader.get_sheet('hoja1')
    accounts_to_collect_sheet = accounts_to_collect_reader.get_sheet('hoja1')

    customer_extractor = tableextraction.DataExtractor(
        input_customers_filename,
        'hoja1',
        {},
        customer_row_extractor
    )

    customers = customer_extractor.extract()

    collections_unpacker = tableextraction.TableUnpacker(collections_sheet)

    for row_unpacker in collections_unpacker.read_rows(2):
        collection = dict()

        document = row_unpacker.get_value_at(2)
        raw_code = row_unpacker.get_value_at(5) or ""
        date = row_unpacker.get_value_at(1)
        customer = row_unpacker.get_value_at(3)
        amount = row_unpacker.get_value_at(4)

        version = HISTORICO
        sales_order = ""
        payments = ""

        if "-" in raw_code:
            version = NUEVO
            order_and_payments = raw_code.split("-")
            sales_order, *payments = order_and_payments

        if sales_order and len(sales_order) != 5:
            error = "Cobranza con orden de compra errónea (%s). Documento: %s" % (sales_order, document)
            errors.append(error)

        collection['version'] = version
        collection['date'] = date
        collection['customer'] = customer
        collection['amount'] = amount
        collection['sales_order'] = sales_order
        collection['payments'] = payments

        collections.append(sales_order, collection)
        collections_for_customers.append(customer, collection)

    bills_unpacker = tableextraction.TableUnpacker(pending_bills_sheet)

    for row_unpacker in bills_unpacker.read_rows(2):

        document_type = row_unpacker.get_value_at(3)
        document = row_unpacker.get_value_at(4)

        if document_type == "Factura":
            document_type = "Factura de Venta"

        document_type_and_number = "%s N° %s" % (document_type, document)

        raw_code = row_unpacker.get_value_at(11)
        amount = row_unpacker.get_value_at(18)

        if not raw_code:
            errors.append("Factura sin descripción. Documento: %s" % document)
            continue

        _, _, sales_order, payment_code, *_ = raw_code.split("-")

        if "de" in payment_code:
            continue

        if not sales_order:
            error = "Factura sin orden de compra. Documento: %s" % document
            errors.append(error)
            continue

        version = NUEVO

        if len(sales_order) != 5:
            error = "Factura con orden de compra errónea (%s). Documento: %s" % (sales_order, document)
            errors.append(error)

        if document_type_and_number in bills and version == NUEVO:
            errors.append("Orden de compra repetida. Documento: %s. Orden de compra: %s" % (document, sales_order))
            continue

        bills[document_type_and_number] = amount

    accounts_to_collect_unpacker = tableextraction.TableUnpacker(accounts_to_collect_sheet)

    for row_unpacker in accounts_to_collect_unpacker.read_rows(2):

        document_date = row_unpacker.get_value_at(1)
        due_date = row_unpacker.get_value_at(2)
        document = row_unpacker.get_value_at(3)
        customer = row_unpacker.get_value_at(4)
        raw_code = row_unpacker.get_value_at(9)

        if "Cobranza" in document: continue
        if not raw_code: continue

        line_amount = float(row_unpacker.get_value_at(8))
        line_balance = row_unpacker.get_value_at(8)

        customer_data = customers[customer]

        account_receivable = AccountReceivable(document_date, due_date, document,
                                               customer, raw_code, customer_data, line_amount,
                                               line_balance, calendar_ops)

        if not account_receivable.is_historic_and_due_for(last_date_of_month):
            continue

        if not account_receivable.validate_payment_plan(errors):
            continue


        account_receivable.configure_previous_collections(collections, collections_for_customers)
        account_receivable.compute_last_collection_date()


        account_receivable.compute_due_payment(bills, first_day_of_current_month, last_date_of_month)

        account_receivable.validate_low_advance_payment(100, errors)
        account_receivable.validate_advance_payment(errors)

        account_receivable.add_to_list_if_advance_payments(list_of_advance_payments)

        if not account_receivable.validate_total_sale_amount_and_plan_value(errors):
            continue

        if not account_receivable.has_due_payments(customers_without_payments_due):
            continue

        if not account_receivable.validate_person(errors):
            continue

        account_receivable.add_to_list_if_in_last_payment(customers_in_last_payment, first_day_of_current_month)
        account_to_collect = account_receivable.to_dict()

        if not account_to_collect['city'] or not account_to_collect['customer']:
            print("FALTA CIUDAD O CLIENTE:", account_to_collect['city'], account_to_collect['customer'])

        accounts_to_collect[account_to_collect['account']].append(account_to_collect)

    for error in errors:
        print("ADVERTENCIA:", error)

    sorted_accounts_C = sorted(accounts_to_collect['C'],
                               key=lambda x: (x['city'], x['customer'], x['order'], x['due_date_datetime']))

    sorted_accounts_D = sorted(accounts_to_collect['D'],
                               key=lambda x: (x['city'], x['customer'], x['order'], x['due_date_datetime']))
    sorted_accounts_D_H = filter(lambda x: x['person'] == 'H', sorted_accounts_D)
    sorted_accounts_D_F = filter(lambda x: x['person'] == 'F', sorted_accounts_D)

    # print("WRONG PERSON ----- ",list(filter(lambda x: x['person'] not in ['H', 'F'], sorted_accounts_D)))

    sorted_accounts_I = sorted(accounts_to_collect['I'],
                               key=lambda x: (x['city'], x['customer'], x['order'], x['due_date_datetime']))

    # crear excel de cobranzas
    collections_filename = outputs_path + 'cuentas_a_cobrar_' + time.strftime("%Y%m%d-%H%M%S") + '.xlsx'
    collections_excelwriter = exceladapter.ExcelWriter(collections_filename)

    columns_config = get_columns_configuration()

    collections_builder_C = excelbuilder.BasicBuilder(sorted_accounts_C, columns_config)
    collections_excelwriter.build_sheet("Créditos", collections_builder_C.build_sheet_data())

    collections_builder_DH = excelbuilder.BasicBuilder(sorted_accounts_D_H, columns_config)
    collections_excelwriter.build_sheet('Débitos Horacio', collections_builder_DH.build_sheet_data())

    collections_builder_DF = excelbuilder.BasicBuilder(sorted_accounts_D_F, columns_config)
    collections_excelwriter.build_sheet('Débitos Facundo', collections_builder_DF.build_sheet_data())

    collections_builder_I = excelbuilder.BasicBuilder(sorted_accounts_I, columns_config)
    collections_excelwriter.build_sheet('ICBC', collections_builder_I.build_sheet_data())

    collections_excelwriter.save()

    last_payment_filename = outputs_path + 'última_cuota_' + time.strftime("%Y%m%d-%H%M%S") + '.xlsx'

    last_payment_composer = exceladapter.ExcelBasicComposer(
        last_payment_filename,
        customers_in_last_payment,
        get_last_payment_columns(),
        "Última Cuota"
    )
    last_payment_composer.save()


    no_payment_due_filename = outputs_path + 'ordenes_de_compra_abiertas_sin_cuotas_a_cobrar_este_periodo' + time.strftime(
        "%Y%m%d-%H%M%S") + '.xlsx'

    no_payment_due_composer = exceladapter.ExcelBasicComposer(
        no_payment_due_filename,
        customers_without_payments_due,
        get_no_payment_due_columns(),
        "Sin cuotas"
    )
    no_payment_due_composer.save()

    advance_payments_filename = outputs_path + 'anticipos_' + time.strftime("%Y%m%d-%H%M%S") + '.xlsx'

    advance_payments_composer = exceladapter.ExcelBasicComposer(
        advance_payments_filename,
        list_of_advance_payments,
        get_advance_payments_columns(),
        "Anticipos"
    )
    advance_payments_composer.save()

    errors_filename = outputs_path + 'errors_' + time.strftime("%Y%m%d-%H%M%S") + '.txt'
    with open(errors_filename, 'w') as f:
        for error in errors:
            f.write(error)
            f.write("\n")

    for payment_error in PAYMENT_ERRORS:
        print(payment_error)


if __name__ == "__main__":
    main()



def get_columns_configuration():
    config_list = list()
    config_list.append(("A", "Ciudad", 'city'))
    config_list.append(("B", "Cliente", 'customer'))
    config_list.append(("C", "Dirección", 'address'))
    config_list.append(("D", "Teléfono", 'phone'))

    config_list.append(("E", "Fecha de Compra", 'date_of_purchase'))
    config_list.append(("F", "Fecha de Vencimiento", 'due_date'))

    config_list.append(("G", "Orden de Compra", 'order'))
    config_list.append(("H", "Última Cobranza", 'last_collection'))

    config_list.append(("I", "Cuotas", 'plan'))

    config_list.append(("J", "Cuotas a pagar", 'current_payment'))
    config_list.append(("K", "Valor de cuota", 'payment'))

    config_list.append(("L", "Monto total a cobrar", 'amount_to_collect'))
    config_list.append(("M", "Descripción", 'full_description'))
    return config_list


def customer_row_extractor(results, unpacker):
    result = dict()

    name = unpacker.get_value_at(1)

    result['address'] = unpacker.get_value_at(8)
    result['city'] = unpacker.get_value_at(29)
    result['phone'] = unpacker.get_value_at(12)

    results[name] = result


NUEVO = "nuevo"
HISTORICO = "histórico"

class AccountReceivable:

    def __init__(self, document_date, due_date, document,
                 customer, raw_code, customer_data, line_amount,
                 line_balance, calendar_operations):
        # line_amount = float(row_unpacker.get_value_at(8))
        # line_balance = row_unpacker.get_value_at(8)


        self.overdue_balance = 0
        self.past_due_debt = ""
        self.collections_for_order = []
        self.collections_for_customer = []
        self.last_collection_date = "Sin cobranza previa"
        self.current_payment_number = 1

        # inject calendar operations
        self.calendar_operations = calendar_operations

        self.document_date = document_date
        self.due_date = due_date
        self.document = document
        self.customer = customer
        self.raw_code = raw_code
        self.line_amount = line_amount
        self.line_balance = line_balance

        self.city = customer_data['city']
        self.phone = customer_data['phone']
        self.address = customer_data['address']

        self._process_version()
        self._process_collection_codes()


    def _process_version(self):
        self.version = HISTORICO  if ("de" in self.raw_code.split("-")[3]) else NUEVO

    def _process_collection_codes(self):
        self.list_of_codes = self.raw_code.split("-")
        self.collection_account, self.collection_person, self.sales_order, *_ = self.list_of_codes

    def configure_previous_collections(self, collections, collections_for_customers):
        if not self.sales_order:
            return

        if self.sales_order in collections:
            self.collections_for_order = collections[self.sales_order]

        if self.customer in collections_for_customers:
            self.collections_for_customer = collections_for_customers[self.customer]


    def compute_last_collection_date(self):
        if not self.sales_order:
            return

        if len(self.collections_for_order) > 0:
            self.last_collection_date = sorted(
                self.collections_for_order,
                key=lambda x: x['date'],
                reverse=True
            )[0]['date'].strftime("%d/%m/%Y")
            return

        if len(self.collections_for_customer) > 0:
            self.last_collection_date = sorted(
                self.collections_for_customer,
                key=lambda x: x['date'],
                reverse=True
            )[0]['date'].strftime("%d/%m/%Y")
            return


    def compute_due_payment(self,bills, first_day_of_current_month, last_date_of_month):
        {
            HISTORICO: self._compute_historic_due_payment,
            NUEVO: self._compute_actual_due_payment
        }[self.version](bills, first_day_of_current_month, last_date_of_month)

    def _compute_historic_due_payment(self, bills, first_day_of_current_month, last_date_of_month):
        self.debt_balance = ""
        self.advance_payment = ""

        self.current_payment_number, self.plan = list(map(lambda x: int(x), self.list_of_codes[3].split(" de ")))
        self.payment_amount = self.line_amount
        self.total_purchase_value = self.payment_amount * int(self.plan)

        self.current_payment_description = str(self.current_payment_number)

    def _compute_actual_due_payment(self, bills, first_day_of_current_month, last_date_of_month):
        self.plan = int(self.list_of_codes[3])

        self.payment_amount = float(self.list_of_codes[4])
        self.debt_balance = self.line_balance

        self.total_purchase_value = bills[self.document]
        self.paid_amount = self.total_purchase_value - self.debt_balance
        self.advance_payment = self.total_purchase_value - (self.plan * self.payment_amount)

        past_payments = 0

        if self.paid_amount > self.advance_payment:
            past_payments = int((self.paid_amount - self.advance_payment) // self.payment_amount)

        self.due_dates = self.calendar_operations.list_of_due_date(self.due_date, self.plan)

        self.due_payments = next((
            self.plan - i for i, v in enumerate(
            reversed(self.due_dates)) if v < first_day_of_current_month),
                            0)
        self.due_payments_with_current = next(
            (self.plan - i for i, v in enumerate(reversed(self.due_dates)) if v <= last_date_of_month), 0)

        self.current_payment_number_by_date = self.current_payment_number = next(
            (self.plan - i for i, v in enumerate(reversed(self.due_dates)) if v <= last_date_of_month), 0)

        self.past_due_debt = self.advance_payment + (self.due_payments * self.payment_amount)

        self.overdue_balance = self.past_due_debt - self.paid_amount

        self.overdue_advance_payment_amount = (self.advance_payment - self.paid_amount) if self.advance_payment - self.paid_amount > 0 else 0

        self.missing_payments = self.due_payments_with_current - past_payments

        if not self.missing_payments > 0:
            return

        self.missing_payment_numbers = list(self.current_payment_number_by_date - x for x in range(self.missing_payments))

        if(len(self.missing_payment_numbers) == 0):
            print(self.current_payment_number_by_date, self.missing_payments)

        self.partial_debt_in_payment = self.overdue_balance % self.payment_amount

        self.partial_debt_in_payment_without_advance = self.partial_debt_in_payment - self.overdue_advance_payment_amount

        if self.partial_debt_in_payment_without_advance > 0:
            first_payment_description = "+$%s de cuota %s" % (
                int(self.partial_debt_in_payment_without_advance),
                self.missing_payment_numbers[-1])
            self.missing_payment_numbers[-1] = first_payment_description

        if self.overdue_advance_payment_amount > 0:
            advance_payment_description = "+$%s de anticipo" % (
                int(self.overdue_advance_payment_amount)
            )
            self.missing_payment_numbers.append(advance_payment_description)

        self.current_payment_description = ", ".join(str(i) for i in self.missing_payment_numbers)


    # continue if is not true
    def is_historic_and_due_for(self, last_date_of_month):
        if self.version == NUEVO: return True
        return self.due_date <= last_date_of_month

    def validate_payment_plan(self, errors):
        if (self.version == HISTORICO): return True

        if len(self.list_of_codes) < 5:
            error = "Cuenta a cobrar sin valor de cuota. Documento: %s - Descripción: %s" % (self.document, self.raw_code)
            errors.append(error)
            return False

        return True

    # should continue
    def validate_low_advance_payment(self, tolerance, errors):
        if (self.version == HISTORICO): return True

        if tolerance >= self.advance_payment > 0:
            total_plan_amount = str(int(self.plan * self.payment_amount))
            error = "El valor del anticipo es muy bajo ($ %s). Comprobar que no se trate de un error de carga. " \
                    "%s. Cliente: %s. Orden de Compra: %s. Valor total de Factura: %s. " \
                    "Valor total según plan: %s" % (
                        self.advance_payment, self.document,
                        self.customer, self.sales_order,
                        self.total_purchase_value, total_plan_amount)
            errors.append(error)
            return False

        return True

    # should continue
    def validate_advance_payment(self,errors):
        if (self.version == HISTORICO): return True

        if self.advance_payment < 0:
            error = "El monto de venta es menor al plan de pagos. " \
                    "Documento: %s - Valor: %s. Plan: %s - Cuota: %s. Diferencia: %s" % (
                       self.document, self.total_purchase_value,
                       self.plan, self.payment_amount,
                       self.advance_payment)
            errors.append(error)
            return False

        return True

    def add_to_list_if_advance_payments(self, list_of_advance_payments):
        if self.advance_payment and self.advance_payment > 0:
            list_of_advance_payments.append({
                "document": self.document,
                "customer": self.customer,
                "total_purchase_value": self.total_purchase_value,
                "advance_payment": self.advance_payment
            })

    def validate_total_sale_amount_and_plan_value(self,errors):
        if (self.version == HISTORICO): return True

        if self.total_purchase_value < (self.plan * self.payment_amount):
            error = "El monto de venta es menor al plan de pagos. " \
                    "Documento: %s - Valor: %s. Plan: %s - Cuota: %s. Diferencia: %s" % (
                        self.document, self.total_purchase_value,
                        self.plan, self.payment_amount,
                        self.advance_payment)
            errors.append(error)
            return False

        return True

    def has_due_payments(self, customers_without_payments_due):
        if (self.version == HISTORICO): return True

        if not self.missing_payments > 0:

            customer_without_payment_due = {
                "city": self.city,
                "customer": self.customer,
                "sales_order": self.sales_order,
                "address": self.address or 'Sin dirección',
                "past_due_debt": self.past_due_debt,
                "paid_amount": self.paid_amount,
                "due_date": self.due_date.strftime("%d/%m/%Y"),
                "raw_code": self.raw_code
            }

            customers_without_payments_due.append(customer_without_payment_due)
            return False

        return True

    def validate_person(self, errors):
        if not self.collection_account in ["I", "C", "D"]:
            error = "Error en el código de cobranza: el primer código debe ser 'I', 'C' o 'D'. Documento: %s - Descripción: %s" % (
            self.document, self.raw_code)
            errors.append(error)
            return False

        if not self.collection_account == "D":
            return True

        if not self.collection_person in ['H','F']:
            error = "Error en el código de cobranza para débito: el segundo código debe ser 'F' o 'H'. Documento: %s - Descripción: %s" % (
            self.document, self.raw_code)
            errors.append(error)

            return False

        return True

    def add_to_list_if_in_last_payment(self, customers_in_last_payment, first_day_of_current_month):
        if self.due_date < first_day_of_current_month:
            return

        if self.plan == self.current_payment_number:

            customer_details = {
                "city": self.city,
                "customer": self.customer,
                "address": self.address or 'Sin dirección'
            }

            customers_in_last_payment.append(customer_details)

    def get_amount_to_collect(self):
        result = float(self.payment_amount) + float(self.overdue_balance)

        if self.version == HISTORICO: return result

        if self.due_payments_with_current == self.due_payments:
            return result - float(self.payment_amount)

        return result

    def to_dict(self):
        account_to_collect = {}

        document_date_str = self.document_date.strftime("%d/%m/%Y")
        due_date_str = self.due_date.strftime("%d/%m/%Y")

        amount_to_collect = self.get_amount_to_collect()

        account_to_collect['version'] = self.version

        account_to_collect['document'] = self.document
        account_to_collect['date_of_purchase'] = document_date_str

        account_to_collect['due_date_datetime'] = self.due_date
        account_to_collect['due_date'] = due_date_str

        account_to_collect['customer'] = self.customer

        account_to_collect['city'] = self.city
        account_to_collect['address'] = self.address
        account_to_collect['phone'] = self.phone

        account_to_collect['account'] = self.collection_account
        account_to_collect['person'] = self.collection_person

        account_to_collect['order'] = self.sales_order

        account_to_collect['last_collection'] = self.last_collection_date
        account_to_collect['total_purchase_value'] = self.total_purchase_value
        account_to_collect['plan'] = self.plan
        account_to_collect['advance_payment'] = self.advance_payment
        account_to_collect['debt_balance'] = self.debt_balance

        account_to_collect['current_payment'] = self.current_payment_description
        account_to_collect['payment'] = self.payment_amount
        account_to_collect['past_due_debt'] = self.past_due_debt
        account_to_collect['overdue_balance'] = self.overdue_balance

        account_to_collect['amount_to_collect'] = amount_to_collect

        account_to_collect['full_description'] = self.raw_code

        return account_to_collect


def get_last_payment_columns():
    config_list = list()
    config_list.append(("A", "Ciudad", 'city'))
    config_list.append(("B", "Cliente", 'customer'))
    config_list.append(("C", "Dirección", 'address'))
    return config_list


import credisur.exceladapter as exceladapter
from . import TableUnpacker


class DataExtractor:

    def __init__(self, filename, sheetname, results_initialization, extract_row, initial_row = 2):
        reader = exceladapter.excelreader.ExcelReader(filename)
        sheet = reader.get_sheet(sheetname)

        self.unpacker = TableUnpacker(sheet)
        self.results = results_initialization
        self.extract_row = extract_row
        self.initial_row = initial_row

    def extract(self):
        for row_unpacker in self.unpacker.read_rows(self.initial_row):
            self.extract_row(self.results, row_unpacker)

        return self.results
